/**
 * Copyright Universidad de Valladolid 2024
 */
package es.markse;
import java.util.ArrayList;

/**
 * Implementacion de la clase abstracta TrayectoSimple
 * @author javcalv
 * @author victorm
 */
public abstract class Combinado extends Trayecto {
	
	//ArrayList para guardar todos los trayectos simples
    protected ArrayList<Simple> trayectosSimples;

    /**
     * Constructor de la clase abstracta TrayectoCombinado
     * @param muelleOrigen Muelle desde el que empieza el Trayecto
	 * @param puertoOrigen Puerto desde el que empieza el Trayecto
	 * @param fechaInicioTrayecto Fecha en la que comienza el Trayecto
	 * @param muelleDestino Muelle en el que finaliza un Trayecto
	 * @param puertoDestino Puerto en el que finaliza un Trayecto
	 * @param fechaFinTrayecto Fecha en la que finaliza un Trayecto
     */
    public Combinado(Muelle muelleOrigen, Puerto puertoOrigen, Fecha fechaInicioTrayecto, Muelle muelleDestino, Puerto puertoDestino, Fecha fechaFinTrayecto) {
        super(muelleOrigen, puertoOrigen, fechaInicioTrayecto, muelleDestino, puertoDestino, fechaFinTrayecto);
        this.trayectosSimples = new ArrayList<>();
    }
    
    /*********************************************************
     * METODOS EJECUTADOS POR EL COSTRUCTOR DE LAS SUBCLASES *
     *********************************************************/
    protected boolean estanVacios(ArrayList<Simple> ts) {
    	return (ts.isEmpty() || ts == null) ? true : false;
    }
    
    /**
     * Metodo que comprueba si hay trayectos de camion en el pack
     * @param trayectos ArrayList de los trayectos simples a comprobar
     * @return true si hay trayectos de camion o false si no los hay
     */
    protected boolean comprobarSiHayTrayectosCamion(ArrayList<Simple> ts) {
    	int trayectosCamiones = 0;
    	for (Simple t : ts) {
    		if (t instanceof TCamion) trayectosCamiones++;	
    	}
    	return (trayectosCamiones == 0)? false : true;
    }
    
    /**
     * Metodo que comprueba si el arraylist tiene un orden correcto:
     * - El destino del muelle/puerto es el origen del trayecto siguiente
     * @param ts ArrayList de los trayectos
     * @return true si es un orden correcto o false si no lo es
     * @throws IllegalArgumentException Si hay elementos vacios de poer medio
     */
    protected boolean ordenTrayectosCorrecto(ArrayList<Simple> ts) {
    	for (int i = 0; i<(ts.size() - 1); i++) {
    		Simple t1 = ts.get(i);
    		Simple t2 = ts.get(i + 1);
    		if (!comprobarDestinoOrigenIguales(t1, t2))
    			return false;
    	}
    	return true;
    }
    
    /**
     * Metodo que comprueba si 2 trayectos (puerto/muelle) son iguales
     * @param t1 trayecto simple primero
     * @param t2 trayecto simple segundo
     * @return true si el destino y el origen de siguiente son iguales o false si no lo son
     */
    private boolean comprobarDestinoOrigenIguales(Simple t1, Simple t2) {
    	return (t1.getMuelleDestino().getIdentificador().equals(t2.getMuelleOrigen().getIdentificador()) && 
    			t1.getPuertoDestino().identificadorPuerto().equals(t2.getPuertoOrigen().identificadorPuerto()))
    			? true : false;
    }
    
    /**
     * Metodo que compara si el puerto/muelle/fecha origen y destino, son iguales a las del primer trayecto
     * y ultimo trayecto de la listas del trayecto del pack repectivamente
     * @param simples Lista de los trayectos (combinados)
     * @throws IllegalArgumentException si el puerto/muelle/fecha origen no coinciden con los de la primera lista del pack de trayectos
     * @throws IllegalArgumentException si el puerto/muelle/fecha destino no coinciden con los de el ultimo de lista del pack de trayectos
     */
    protected void compararTrayectosDeLaLista(ArrayList<Simple> simples) {
    	Simple primerT = this.trayectosSimples.get(0);
    	Simple ultimoT = this.trayectosSimples.get(this.trayectosSimples.size() -1);
    	
    	//Comparamos los datos de origen si son iguales que los del primer trayecto del pack
    	if (!(this.muelleOrigen.getIdentificador().equals(primerT.getMuelleOrigen().getIdentificador()) &&
    		this.puertoOrigen.identificadorPuerto().equals(primerT.getPuertoOrigen().identificadorPuerto())&&
    		this.fechaInicioTrayecto == primerT.getFechaInicioTrayecto())) 
    			throw new IllegalArgumentException("El puerto/muelle/fecha origen no coinciden con los de la primera lista del pack de trayectos");
    	
    	//Comparamos los datos de destino si son iguales que los del ultimo trayecto del pack
    	if (!(this.muelleDestino.getIdentificador().equals(ultimoT.getMuelleDestino().getIdentificador()) &&
        		this.puertoDestino.identificadorPuerto().equals(ultimoT.getPuertoDestino().identificadorPuerto())&&
        		this.fechaFinTrayecto == ultimoT.getFechaFinTrayecto())) 
        			throw new IllegalArgumentException("El puerto/muelle/fecha destino no coinciden con los de el ultimo de lista del pack de trayectos");
    }
}
