/**
 * Copyright UVa 2024/2025
 */
package es.markse;
import java.util.ArrayList;
import java.util.List;

/**
 * Implementacion de la clase Contenedor
 * @author javcalv
 * @author victorm 
 */
public abstract class Contenedor {
	protected final List<Trayecto> trayectos = new ArrayList<>();
	
	private ISO6346 codigo;
	private float pesoTara;
	private float maximaCargaUtil;
	private float volumen;
	protected boolean techo;
	
	//Seleccion del estado y de las medidas
	public enum estados{ REGOGIDA, TRANSITO }
	public enum pesos{ KILOS, LIBRAS}
	public enum volumenes{ METROS, PIES }
	private estados estadoActual;
	private final pesos pesoSeleccionado;
	private final volumenes volumenSeleccionado;
	
	//Parametros para el paso de unas medidas a otras
	private final float KILOS_A_LIBRAS = 2.20462f;
	private final float LIBRAS_A_KILOS = 0.453592f;
	private final float METROS_A_PIES = 35.3147f;
	private final float PIES_A_METROS= 0.0283168f;

	/**
	 * Constructor del Objeto Contenedor
	 * @param pesoTara El peso del contenedor en kilogramos
	 * @param maximaCargaUtil La carga util del contenedor dada en kilogramos
	 * @param volumen El volumen dado en metros cúbicos
	 * @param estadoActual estado del contenedor
	 * @param pesoSeleccionado en que medidas se dan los pesos de la tara y la carga
	 * @param volumenSeleccionado en que medida se da el volumen
	 * @param techo indica si tiene techo (true) o no (false)
	 */
	public Contenedor(ISO6346 codigo, float pesoTara, float maximaCargaUtil, float volumen, estados estadoActual, pesos pesoSeleccionado, volumenes volumenSeleccionado) {
        if (pesoTara<1 || maximaCargaUtil <1 || volumen<1)
        	throw new IllegalArgumentException("Los pesos no puedne ser menores a 0");
        this.codigo = codigo;
        
		this.pesoTara = pesoTara;
		this.maximaCargaUtil = maximaCargaUtil;
		this.volumen = volumen;
		
		this.estadoActual = estadoActual;
		this.pesoSeleccionado = pesoSeleccionado;
		this.volumenSeleccionado = volumenSeleccionado;
	}
	
	/**
	 * 1.1.13 Metodo que sirve para cambiar el estado a recogida de un contenedor
	 */
	public void cambiarEstadoARecogida() {
		this.estadoActual = estados.REGOGIDA;
	}
	
	/**
	 * 1.1.14 Metodo que sirve para cambiar un contenedor a estado de transito
	 */
	public void cambiarEstadoATransito() {
		this.estadoActual = estados.TRANSITO;
	}
	
	/**
	 * 1.1.16 Metodo que devuelve el volumen del contenedor en metros cubicos
	 * @return volumen en metros cubicos
	 */
	public double volumenEnMetrosCubicos() {
		return (volumenSeleccionado == volumenes.METROS) ? this.volumen : this.volumen*this.PIES_A_METROS;	
	}
	
	/**
	 * 1.1.17 metodo que devuelve el volumen del contenedor en pies cubicos
	 * @return volumen en pies cubicos
	 */
	public double volumenEnPiesCubicos() {
		return (volumenSeleccionado == volumenes.PIES) ? this.volumen : this.volumen*this.METROS_A_PIES;
	}
	
	/**
	 * 1.1.18 Metodo que devuelve el peso del contenedor en Kilogramos
	 * @return peso del contenedor en kilogramos
	 */
	public double obtenerPesoKilos() {
		return (this.pesoSeleccionado == pesos.KILOS ) ? this.pesoTara : this.pesoTara *this.LIBRAS_A_KILOS;
	}
	
	/**
	 * 1.1.19 Metodo que devuelve el peso del contenedor en Libras
	 * @return peso del contenedor en Libras
	 */
	public double obtenerPesoLibras() {
		return (this.pesoSeleccionado == pesos.LIBRAS ) ? this.pesoTara : this.pesoTara *this.KILOS_A_LIBRAS;
	}
	
	/**
	 * Metodo que calcula el costo total de los trayectos de un contenedor
	 * @param costeDiarioTrayecto El coste diario de un dia de trayecto
	 * @param costePorMilla El coste por milla recorrida en un trayecto
	 * @return el precio total del transporte del contenedor
	 */
	public double precioTransporteTotalContenedor() {
		if (this.trayectos.isEmpty())
			return 0.0;
		double precioTotal = 0.0;
		for (Trayecto t : this.trayectos) {
			precioTotal += t.precioTrayectoEnEuros();
		}
		return precioTotal;
	}
	
	
	/******************************
	 * EMPLEACION DE OTROS METODOS*
	 ******************************/
	
	/**
	 * Metodo que sirve para anyadir un trayecto al contenedor
	 * @param t El trayecto que se añade al contenedor
	 * @throws IllegalArgumentException si es trayecto es nulo, 
	 * @throws IllegalStateException si la fecha fin del ultimo trayecto es superior a la de inicio del nuevo trayecto
	 * @throws IllegalStateException si el puerto/muelle destinos del ultimo trayecto no son los de origen del nuevo.
	 */
	public abstract void anyadirTrayecto(Trayecto t);

	protected void comprobarDatosContenedor(Trayecto t) {
		Trayecto ultimoT = this.trayectos.get(this.trayectos.size() -1);
		//Si la fecha fin es mayor a la de inicio del nuevo
		if (ultimoT.getFechaFinTrayecto().getDiasDesdeEpoch() > t.getFechaInicioTrayecto().getDiasDesdeEpoch())
			throw new IllegalStateException("La fecha fin del ultimo trayecto no puede ser mayor la fecha de inicio del siguiente");
		//Si el puerto destino no es el origen del nuevo
		if (!ultimoT.getPuertoDestino().identificadorPuerto().equals(t.getPuertoOrigen().identificadorPuerto()))
			throw new IllegalStateException("El puerto de origen debe ser el mismo que el de destino del ultimo trayecto");
		//Si el muelle de destino no es el de origen del nuevo
		if (!ultimoT.getMuelleDestino().getIdentificador().equals(t.getMuelleOrigen().getIdentificador()))
			throw new IllegalStateException("El Muelle de origen debe de ser igual al de destino de su ultimo trayecto");
		this.trayectos.add(t);	
	}
	
	/**
	 * Metodo que devuelve si un contenedor tiene techo
	 * @return si tiene techo (true) o si no lo tiene (false)
	 */
	public abstract boolean tieneTecho();
	
	/**
	 * Metodo que devuelve la maxima carga util de un contenedor
	 * @return La maxima carga util de un contenedor
	 */
	public float cargaUtilContenedorKilos() {
		return (this.pesoSeleccionado == pesos.KILOS ) ? this.maximaCargaUtil : this.maximaCargaUtil *this.LIBRAS_A_KILOS;
	}
	
	/**
	 * Metodo que devuelve la maxima carga util de un contenedor
	 * @return La maxima carga util de un contenedor
	 */
	public float cargaUtilContenedorLibras() {
		return (this.pesoSeleccionado == pesos.LIBRAS ) ? this.maximaCargaUtil : this.maximaCargaUtil *this.KILOS_A_LIBRAS;
	}
	
	/**
	 * Metodo que sirve para devolver el codigo del contenedor
	 * @return el codigo del contenedor (compelto)
	 */
	public String getCodigo() {
		return this.codigo.codigoContenedor();
	}
	
	/**
	 * Metodo que sirve para devolver si el contenedor esta en transito
	 * @return true si esta en transito o false si no lo esta
	 */
	public boolean contenedorEnTransito() {
		return (estadoActual == estados.TRANSITO) ? true : false;
	}
	
	/**
	 * Metodo que sirve para devolver si el contenedor esta en recogida
	 * @return true si esta en recogida o false si no lo esta
	 */
	public boolean contenedorEnRecogida() {
		return (estadoActual == estados.REGOGIDA) ? true : false;
	}
	
}