package es.markse;

/**
 * Implementación de la fecha del calendario gregoriano.
 * Ejercicio de Programación Orientada a Objetos
 * Implementación basada en tres atributos
 * @author felix
 * @author marga
 */
public class Fecha {
	private int anno;
	private int mes;
	private int dia;

	/**
	 * Inicialización a partir de número de días desde «Epoch».
	 * 
	 * Utilizamos la notación habitual Epoch para referirnos al 1-1-1970, al estilo
	 * *N?X. No comprueba que el día es correcto.<br>
	 * @param dDías transcurridos desde 1 de enero de 1970, 
	 * @throws IllegalArgumentException cuando {@code d<0}
	 */
	public Fecha(int d) {
		if (d<0)
			throw  new IllegalArgumentException("Fecha incorrecta");

		Fecha f = new Fecha(1, 1, 1970);
		for (int i = 1; i <= d; i++) {
			f = f.mañana();
		}
		anno = f.getAnno();
		mes = f.getMes();
		dia = f.getDia();
	}

	/**
	 * Inicialización a partir de un día, mes y año.
	 * 
	 * 
	 * @param d
	 *          Día del mes, entre 1 y el número de días del {@code mes}
	 * @param m
	 *          Mes, entre 1 y 12
	 * @param a
	 *          Año con cuatro cifras, no negativo
	 * @throws IllegalArgumentException cuando la d, m o a no corresponda a una fecha correcta
	 */
	public Fecha(int d, int m, int a) {
		if (!esFechaCorrecta(d,m,a))
				throw  new IllegalArgumentException("Fecha incorrecta");
		dia = d;
		mes = m;
		anno = a;
	}

	/**
	 * Días transcurridos desde el 1 de enero de 1970.
	 * 
	 * @return Negativo para fechas anteriores a 1-1-1970
	 */
	public int getDiasDesdeEpoch() {
		return diferencia(new Fecha(1, 1, 1970));
	}

	/**
	 * Número de día del mes.
	 * 
	 * @return entre 1 y número de días de {\@code getMes()}
	 */
	public int getDia() {
		return dia;
	}

	/**
	 * Número del meses.
	 * 
	 * @return Entre 1 y 12
	 */
	public int getMes() {
		return mes;
	}

	/**
	 * Año.
	 * 
	 * @return Año con 4 cifras y positivo
	 */
	public int getAnno() {
		return anno;
	}

	/**
	 * El día de la semana.
	 * 
	 * La semana empieza el lunes
	 * 
	 * @return día de la semana entre (1=lunes,...,7=domingo)
	 */
	public int diaSemana() {
		// El 1 de enero de 1970 fue jueves
		return (getDiasDesdeEpoch()+3) % 7 + 1 ;
	}

	/**
	 * ¿Es día de diario?
	 * 
	 * @return Cierto de lunes a viernes, falso en otro caso
	 */
	public boolean esLaborable() {
		return (diaSemana() >= 1) && (diaSemana() <= 5);
	}

	/**
	 * El mañana de {@code this}.
	 * 
	 * @return Mañana
	 */
	public Fecha mañana() {
		int d = getDia();
		int m = getMes();
		int a = getAnno();
		d = d + 1;
		if (d > diasMes(m, a)) {
			d = 1;
			m = m + 1;
			if (m == 13) {
				m = 1;
				a = a + 1;
			}
		}
		return new Fecha(d, m, a);

	}

	/**
	 * El ayer de {@code this}.
	 * 
	 * @return Ayer
	 */
	public Fecha ayer() {
		int d = getDia();
		int m = getMes();
		int a = getAnno();
		d = d - 1;
		if (d == 0) {
			m = m - 1;
			if (m == 0) {
				m = 12;
				a = a - 1;
			}
			d = diasMes(m, a);
		}
		return new Fecha(d, m, a);

	}

	/**
	 * Diferencia en días entre {@code this} y {@code other}.
	 * 
	 * @param otro
	 *          día a restar
	 * @return {@code this - otro}
     * @throws IllegalArgumentException cuando {@code otro==null}
	 */
	public int diferencia(Fecha otro) {
        if (otro==null)
            throw new IllegalArgumentException("LLamada incorrecta: otro==null");
        
		Fecha inicio, fin, contador;
		int signo = comparaCon(otro);
		switch (signo) {
		case 1:
			inicio = otro;
			fin = this;

			break;
		case -1:
			inicio = this;
			fin = otro;
			break;
		default: // Las dos fechas son iguales
			return 0;
		}
		contador = inicio;
		int dias = 0;
		while (!contador.igualA(fin)) {
			contador = contador.mañana();
			dias += 1;
		}
		return signo * dias;
	}

	/**
	 * Incremento en un número de días.
	 * 
	 * @param n Número de días en que se incrementa, debe ser un número positivo
	 * @return {@code this + n}
	 * @throws IllegalArgumentException cuando {@code n>=0}
	 */
	public Fecha mas(int n) {
		if (n<=0)
			throw new IllegalArgumentException("Llamada incorrecta: n >=0");
		
		Fecha f = new Fecha(getDia(), getMes(), getAnno());
		for (int i = 1; i <= n; i++) {
			f = f.mañana();
		}
		return f;
	}

	/**
	 * Número de la semana según la norma ISO.
	 * 
	 * La semana empieza el lunes y termina el domingo. El número de semana es el
	 * número de orden para determinar una semana exacto durante el año. De acuerdo
	 * con la norma ISO 8601 y NEN 2772 es la primera semana del año en que contiene
	 * cuatro o más días de ese año. Porque, según NEN 2772, el primer dia de la
	 * semana es un Lunes, se puede decir que la semana 1 es la semana que lleva el
	 * primer Jueves del año. La primera semana del año tambien es la semana que
	 * conlleva la fecha 4 de enero, o la semana con la mayoria de los dias.
	 * 
	 * @return Número de semana entre 1 y TODO Determinar
	 */
	public int semana() {
		// TODO Implementar número de semana
		return 0;
	}

	/**
	 * Comparación habitual entre fechas.
	 * <ul>
	 * <li>a {@literal >} b cuando a.compareTo(b) {@literal >} 0 <br>
	 * <li>a {@literal <} b cuando a.compareTo(b) {@literal <} 0<br>
	 * <li>a {@literal >}= b cuando a.compareTo(b) {@literal >}= 0<br>
	 * <li>a {@literal <}= b cuando a.compareTo(b) {@literal <}= 0<br>
	 * </ul>
	 * 
	 * @param otro
	 *          fecha con la que comparamos
	 * @return -1,0 o 1
     * @throws IllegalArgumentException cuando {@code otro==null}
	 */
	public int comparaCon(Fecha otro) { 
         if (otro==null)
            throw new IllegalArgumentException("LLamada incorrecta: otro==null");
		if (getAnno() != otro.getAnno())
			return Integer.compare(getAnno(), otro.getAnno());
		else if (getMes() != otro.getMes())
			return Integer.compare(getMes(), otro.getMes());
		else if (getDia() != otro.getDia())
			return Integer.compare(getDia(), otro.getDia());
		else
			return 0;
	}

	/** Comparar si una fecha es igual a otra.
	 * ¿Porqué no podemos utilizar equals() como decía alguna de las transparencias
	 * de la asignatura?
	*
	 * @param f
	 *          Fecha con que comparamos this
	 * @return Cierto si los dos objetos representan la misma fecha
     * @throws IllegalArgumentException cuando {@code f==null}
	 */
	public boolean igualA(Fecha f) {
        if (f==null)
            throw new IllegalArgumentException("LLamada incorrecta: f==null");

		return getDia() == f.getDia() && getMes() == f.getMes() && getAnno() == f.getAnno();
	}
	
	/**
	 * Representación elemental en formato numérico
	 * @return La fecha en formato legible, d-m-a
	 */
	public String aCadena() {
		return getDia()+"-"+getMes()+"-"+getAnno();
	}

	/**
	 * Dado un mes y un año, se obtiene el número de los días de ese mes. Se tiene en cuenta que el año sea
	 * bisiesto
	 */ 
	public int diasMes(int m, int a) {
		int resultado = 0;
		switch (m) {
		case 1:
		case 3:
		case 5:
		case 7:
		case 8:
		case 10:
		case 12:
			resultado = 31;
			break;
		case 4:
		case 6:
		case 9:
		case 11:
			resultado = 30;
			break;
		case 2:
			if ((a % 4 == 0) && (a % 100 != 0) || (a % 400 == 0))
				resultado = 29;
			else
				resultado = 28;
		}
		return resultado;
	}

/**
 * Es un método que dado una fecha en formato dia , mes y año, nos devuelve si es correcta o no
 */
private boolean esFechaCorrecta(int dd, int mm, int aa){
	if ((dd>diasMes(mm,aa)) || (dd<1) ||(mm<1) || (mm>12)||(aa<1000)||(aa>9999)){
		return false;
	}else 
		{
		return true;
		}

 }
}