/**
 * Copyright UVa 2024/2025
 */
package es.markse;
import java.util.regex.Pattern;
/**
 * Implementacion de la clase ISO6346 para el codigo del contenedor
 * @author javcalv
 * @authro victorm
 */
public class ISO6346 {
	
	private final String codigoDuenyo;
	private char equipamiento;
	private final int numeroSerie;
	private int digitoControl;
	private final Pattern CODIGO_DUENO = Pattern.compile("[A-Za-z]{3}");
	
	/**
	 * Implementacion de la clase ISO6346
	 * @param codigoDuenyo Codigo del dueño de 3 letras  (Mayusculas)
	 * @param equipamiento Equipamiento del contenedor --> U, J o Z
	 * @param numeroSerie Numero de serie del contenedor. De 0-999999
	 * @throws IllegalArgumentException si el codigo del dueño es nulo
	 * @throws IllegalArgumentException si el codigo del dueño no tiene el patron correcto
	 * @throws IllegalArgumentException si el equipamiento no es correcto
	 * @throws IllegalArgumentException si el numero de serie es incorrecto
	 */
	public ISO6346(String codigoDuenyo, char equipamiento, int numeroSerie) {
		if(codigoDuenyo == null)
			throw new IllegalArgumentException("El codigo del dueño no puede ser nulo");
		if (!validarCodigoDueño(codigoDuenyo))
			throw new IllegalArgumentException("El codigo del dueño no tiene el patron correcto (3 letras)");
		if (!comprobarEquipamiento(equipamiento))
			throw new IllegalArgumentException("Equipamiento debe ser 'U', 'J' o 'Z'");
		if (numeroSerie<0 || numeroSerie>999999)
			throw new IllegalArgumentException("El numero de serie debe ser entre 0-999999");
		
		this.codigoDuenyo = codigoDuenyo.toUpperCase();
		this.equipamiento = equipamiento;
		this.numeroSerie = numeroSerie;
		this.digitoControl = calcularDigitoControl(codigoDuenyo, equipamiento, numeroSerie);
	}
	
	
	/**
	 * Metodo que sirve para validar si el codigo del dueño tiene el patron correcto
	 * @param codigoDuenyo Codigo del dueño que se debe validar
	 * @return true si es correcto o falso si no lo es
	 */
	private boolean validarCodigoDueño(String codigoDuenyo) {
		return (this.CODIGO_DUENO.matcher(codigoDuenyo).matches()) ? true : false;
	}
	
	/**
	 * Metodo que comprueba el equipamiento es correcto
	 * @param equipamiento Equipamiento que se comprobara
	 * @return true si es correcto o false si no lo es
	 */
	private boolean comprobarEquipamiento(char equipamiento) {
		return (equipamiento == 'U' || equipamiento == 'J' || equipamiento == 'Z')? true: false;
	}
	
	/**
	 * Metodo que sirve para calcular el diito de control mediante el codigo del dueño, el equipamiento y el
	 * numero de serie
	 * @param codigoDuenyo Codigo del dueño
	 * @param equipamiento = equipamiento del contenedor
	 * @param numeroSerie numero de serie del contenedor
	 * @return devuelve el digito de control
	 */
	private int calcularDigitoControl(String codigoDueno, char equipamiento, int numeroSerie) {
		String codigoCompleto = codigoDueno + equipamiento + numeroSerieString(numeroSerie);
	    int[] valores = {10, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 34, 35, 36, 37, 38};
	    int suma = 0;
	    for (int i = 0; i < codigoCompleto.length(); i++) {
	        char c = codigoCompleto.charAt(i);
	        int valor;
	        if (Character.isLetter(c))valor = valores[c - 'A']; 
	        else valor = Character.getNumericValue(c);
	        suma += valor*(int) Math.pow(2, i);
	    }
	    int resto = suma % 11;
	    return (resto == 10) ? 0 : resto;
    }
	
	/**
	 * Metodo que pasa el numero de Serie a un String (con los 6 caracteres)
	 * @param numeroSerie Numero de serie a transformar
	 * @return numero de Serie transformado en String
	 */
	private String numeroSerieString(int numeroSerie) {
		return String.format("%06d", numeroSerie);
	}

	/**
	 * Metodo que devuelve el codigo del dueño
	 * @return codigo del dueño
	 */
	public String codigoDuenyo() {
		return this.codigoDuenyo;
	}
	
	/**
	 * Metodo que devuelve el equipamiento del codigo
	 * @return equipamiento
	 */
	public char equipamiento() {
		return this.equipamiento;
	}
	
	/**
	 * Metodo que devuelve el numero de Serie del contenedor
	 * @return numero de Serie del contenedor
	 */
	public int numeroSerie() {
		return this.numeroSerie;
	}
	
	/**
	 * Metodo que devuelve el codigo del Contenedor completo
	 * @return codigo del contenedor
	 */
	public String codigoContenedor() {
		return (this.codigoDuenyo + this.equipamiento + numeroSerieString(numeroSerie) + this.digitoControl);
	}
}