/**
 * Copyright UVa 2024/2025
 */
package es.markse;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

import es.uva.inf.poo.maps.GPSCoordinate;

/**
 * Implementacion de la clase muelle donde se alojaran los Contenedores (Clase)
 * @author javcalv
 * @author victorm
 */
public class Muelle {
	private final Plaza[] plazas;
	private final String identificador;
	public enum estado{
		OPERATIVO, FUERA_DE_SERVICIO
	}
	private estado estadoActual;
	private final GPSCoordinate cord;
	private final Pattern IDENTIFICADOR_REGUEX = Pattern.compile("^\\d{2}$");
	private final Pattern CODIGO_CONTENEDOR = Pattern.compile("[A-Z]{4}\\d{7}");
	
	//Modificaciones Practica 2
	private boolean utilidadTren;
	private boolean alejadoBarco;
	
	private static class Plaza {
		private int altura;
		private final List<Contenedor> contenedores = new ArrayList<>();
		
		/**
		 * Constructor del objeto Plaza
		 * @param altura. Altura maxima apilable de los contenedores
		 */
		public Plaza(int altura){
			this.altura = altura;
		}
		
		/**
		 * 1.1.8 Metodo que devuelve si una plaza esta vacia o no
		 * @return true si esta vacia o false si no lo esta
		 */
		public boolean plazaVacia() {
			return this.contenedores.isEmpty();
		}
		
		/**
		 * 1.1.8 Metodo que devuelve si una plaza esta llena o no.
		 * - Si la cantidad de contenedores es la maxima
		 * - Si el ultimo contenedor no tiene techo
		 * - O si el contenedor es un FlatRack
		 * @return true si la plaza esta llena o si no lo esta
		 */
		public boolean plazaLlena() {
			return (this.contenedores.size() == this.altura()
					|| (!this.contenedores.isEmpty() && !this.ultimoContenedor().tieneTecho()) 
					|| this.contenedores.get(0) instanceof FlatRack) ? true : false;
		}
		
		/**
		 * 1.1.10 Metodo que indica el nivel que tiene un contenedor, dado un codigo.
		 * @param codigo Codigo del contenedo que queremos ver su plaza
		 * @return la altura en la que se encuentra (del 1 que es la baja ahasta n, que es el máximo de contenedores apilabres)
		 * @throws IllegalArgumentException si el codigo es nulo
		 */
		public int nivelContenedor(String codigo) {
			if (codigo == null)
				throw new IllegalArgumentException("El codigo no puede ser nulo");
		    for (int i = 0; i < this.contenedores.size(); i++) {
		        if (this.contenedores.get(i).getCodigo().equals(codigo)) {
		            return i + 1;
		        }
		    }
		    return -1;
		}
		
		/**
		 * 1.1.11 Metodo que sirve para colar el contenedor en una plaza (apilarlo si es posible)
		 * @param codigo: Codigo del contenedor
		 * @return true si se ha colodado, o false si no
		 * @throws IllegalArgumentException si el contenedor es nulo
		 */
		public void colocarContenedor(Contenedor contenedor) {
			//Encapsulamos errores para evitarlos en un futuro (si hay cambios)
			if (contenedor == null)
				throw new IllegalArgumentException("El contenedor no puede ser nulo");
			
			//Si el contenedor esta en transito
			if (contenedor.contenedorEnTransito())
				throw new IllegalStateException("El Contenedor esta en transito, no se puede colocar");
			
			//Si la plaza esta llena
			if(this.plazaLlena())
				throw new IllegalStateException("La plaza está llena");
			
			//Comprobamos si estan vacias o no para un tipo de contenedores
			if ((contenedor instanceof FlatRack || contenedor instanceof Refrigerado) && this.plazaVacia()) {
				this.contenedores.add(contenedor);
			}
			
			else {
				this.contenedores.add(contenedor);
			}
		     
		    
		}
		
		/**
		 * 1.1.12 Metodo que desapila el ultimo contendor
		 */
		public void desapilarContenedor() {
			if(!this.contenedores.isEmpty())
				this.contenedores.remove(contenedores.size() - 1);
		}
		
		/*********************************************
		 * OTROS METODOS DE LA PLAZA PARA SU GESTION *
		 *********************************************/
		
		/**
		 * Metodo que devuelve el numero decontenedores
		 * @return numero de contenedores
		 */
		public int numeroContenedores() {
			return contenedores.size();
		}
		
		/**
		 * Metodo que devuelve la altura maxima de la plaza
		 * @return altura
		 */
		public int altura() {
			return this.altura;
		}
		
		/**
		 * Metodo que devuelve true o false si el contenedor que introducimos se encuentra en el nivel mas alto actual de la plaza, es decir, que
		 * es el ultimo contenedor apilado en esa plaza (sin necesidad que sea el maximo)
		 * @param codigoContenedor Codigo del contenedor que se comprueba si es el ultimo
		 * @return true si es el ultimo o false si no lo es.
		 */
		public boolean estaEnUltimoNivel(String codigoContenedor) {
			Contenedor c = this.ultimoContenedor();
			return (c == null) ? false : c.getCodigo().equals(codigoContenedor);
		}
		
		/**
		 * Metodo que indica si un contenedor se encuentra en la plaza o no
		 * @param codigo Codigo del contenedor que queremos buscar
		 * @return true si contiene ese contenedor en la Plaza o false si no lo tiene
		 */
		public boolean contieneContenedor(String codigo) {
	        return (this.nivelContenedor(codigo)!=-1) ? true : false;
	    }
		
		/**
		 * Metod que devuelve el ultimo contenedor de la Plaza
		 * @return ultimo contenedor de la plaza o null si esta vacio
		 */
		public Contenedor ultimoContenedor() {
			if (contenedores.isEmpty()) {
		        return null;
		    }
			return contenedores.get(contenedores.size() -1);
		}
	}
	
	/**
	 * Constructor del objeto Muelle
	 * @param identificador Identificador de 2 digitos (numeros)
	 * @param cord Punto GPS que lo localiza. Clase GPSCoordinate.
	 * @param estadoActual Estado del muelle.
	 * @param plazas Numero de plazas totales que tiene el Muelle
	 * @param altura numero maximo de contenedores que se pueden apilar encima de otro
	 * @param utilidadTren booleano que indica si un muelle puede usar un tren como transporte
	 * @param alejadoBarco booleano que indica si esta alejado de un barco (puede o no usar ese transporte)
	 * @throws IllegalArgumentException si el identificador o la coordenada es nulo
	 * @throws IllegalArgumentException si el tamaño de la altura y plaza son menores a 1
	 * @throws IllegalArgumentException si el identificador tiene un formato incorrecto
	 */
	public Muelle (String identificador, GPSCoordinate cord, estado estadoActual, int plazas, int altura, boolean utilidadTren, boolean alejadoBarco) {
		if(identificador == null || cord == null)
			throw new IllegalArgumentException("Los valores del identificador y coordenada no pueden ser nulos");
		if(plazas<1 || altura<1)
			throw new IllegalArgumentException("Los tamaños (altura y plaza) no puedne ser menores a 1");
		if (!comprobarIdentificador(identificador))
			throw new IllegalArgumentException("El formato del identificador es incorrecto");
		
		this.identificador = identificador;
		this.cord = cord;
		this.plazas = new Plaza[plazas];
		this.estadoActual = estadoActual;
		//Inicializamos el array de las plazas
		for (int i = 0; i < plazas; i++) {
            this.plazas[i] = new Plaza(altura);
        }
		
		//Modificaciones practica 2
		this.alejadoBarco = alejadoBarco;
		this.utilidadTren = utilidadTren;
	}
	
	/**
	 * Metodo que comprueba si el identificador es correcto
	 * @param identificador Identificador del muelle
	 * @return true si es correcto o false si no lo es
	 */
	private boolean comprobarIdentificador(String identificador) {
		return (IDENTIFICADOR_REGUEX.matcher(identificador).matches()) ? true : false;
	}
	
	/***************************************
	 * EMPLEACION DE LOS METODOS QUE PIDEN *
	 ****************************************/
	
	/**
	 * 1.1.7 Metodo que devuelve el numero de plazas totales que tiene el muelle
	 * @return numero de plazas (totales) del muelle
	 */
	public int numeroDePlazasTotales() {
		return this.plazas.length;
	}
	
	/**
	 * 1.1.8 Metodo que devuelve el numero de plazas vacias, es decir que no se encuentran ocupadas por ningun
	 * contenedor.
	 * @return numero de plazas vacias.
	 */
	public int plazasVacias() {
		int c = 0;
		for (Plaza p : this.plazas)
			if(p.plazaVacia()) c++;
		return c;
	}
	
	/**
	 * 1.1.8 Metodo que devuelve el numero de plazas llenas. Una plaza llena es la que:
	 * - Se encuentran con el numero máximo de contenedores apilados en la plaza
	 * - El contenedor o el ultimo contenedor apilado no tiene techo, lo que no se peden colocar más.
	 * @return numero de plazas llenas
	 */
	public int plazasLlenas() {
		int c = 0;
		for (Plaza p: this.plazas) 
			if (p.plazaLlena()) c++;
		return c;
	}
	
	/**
	 * 1.1.8 Metodo que devuelve el numero de plazas semillenas. Las plazas semillenas son aquellas que
	 * contienen un contenedor y se puede apilar mas encima de el (tiene techo)
	 * @return numero de plazas semillenas
	 */
	public int plazasSemillenas() {
		int c = 0;
		for (Plaza p: this.plazas) 
			if (!p.plazaLlena() && !p.plazaVacia()) c++;
		return c;
	}
	
	/**
	 * 1.1.9 Metodo que devuelve la plaza de un contenedor dentro del muelle.
	 * Las plazas van en orden, del 0 hasta la ultima plaza del muelle disponible
	 * @param codigo: codigo del contenedor para ver su plaza actual
	 * @return la plaza del contenedor o -1 si no se encuentra en ninguna plaza
	 */
	public int plazaActual(String codigo) {
		if(codigo == null)
			throw new IllegalArgumentException("El codigo no puede ser nulo");
		if(!comprobarCodigoContenedor(codigo))
			throw new IllegalArgumentException("El codigo de contenedor es incorrecto");
		
        for (int i = 0; i < this.plazas.length; i++) {
            if (this.plazas[i].numeroContenedores() != 0 && this.plazas[i].contieneContenedor(codigo)) {
                return i;
            }
        }
        return -1;
    }

	/**
	 * 1.1.10 Metodo que devuelve el nivel de la plaza en la que se encuentra un contenedor.
	 * El valor varia entre 1 y el numero máximo de contenedores que se pueden apilar (diferente en cada muelle)
	 * @param codigo: codigo del contenedor que queremos buscar su nivel
	 * @return el nivel en el que se encuentra el contenedor o -1 si no se encuentra
	 */
	public int nivelEnPlaza(String codigo) {
		if(codigo == null)
			throw new IllegalArgumentException("Codigo del contenedor no puede ser nulo");
		if(!this.comprobarCodigoContenedor(codigo))
			throw new IllegalArgumentException("El codigo de contenedor no tiene el formato correcto");
        for (Plaza plaza : this.plazas) {
            if (plaza.numeroContenedores() != 0 && plaza.contieneContenedor(codigo)) {
                return plaza.nivelContenedor(codigo);
            }
        }
        return -1;
    }

	/**
	 *  1.1.11 Metodo para colocar un contenedor en una plaza y apilarlo si es posible
	 * @param codigo Codigo del contenedor que queremos colocar
	 * @param plaza La plaza del contenedor
	 * @throws IllegalArgumentException si el contenedor es nulo
	 * @throws IllegalArgumentException si la plaza esta fuera de rango, 
	 * @throws IllegalStateException si el muelle no esta operativo
	 * @throws IllegalStateException si ese contenedor ya esta apilado
	 * @throws IllegalStateException si ese contenedor esta en transito, 
	 * @throws IllegalStateException si la plaza esta llena
	 * @throws IllegalStateException si la plaza no esta vacia y queremos colocar un contenedor Regrigerado
	 */
	public void colocarContenedorEnPlaza(Contenedor contenedor, int plaza) {
		//Comprobamos si el contenedor es nulo o la plaza no pertenece al rango
		if (contenedor == null) 
			throw new IllegalArgumentException("El Contenedor no puede ser nulo");
		if (plaza < 0 || plaza > this.numeroDePlazasTotales()) 
			throw new IllegalArgumentException("La plaza debe de estar entre 0-"+ (this.plazas.length -1));
		
		//Comprobamos si esta operativo o no
		if(!this.estaOperativo())
			throw new IllegalStateException("El muelle no esta operativo");
			
		//Si esta entransito
		if (contenedor.contenedorEnTransito()){
			throw new IllegalStateException("El Contenedor esta en transito, no se puede colocar");
		}
		
		//Si ya se encuentra apilado el contenedor en el muelle
		if (nivelEnPlaza(contenedor.getCodigo()) != -1) {
			throw new IllegalStateException("El Contenedor ya esta apilado");
		}
		
	    //Verificar si la plaza esta llena
		Plaza p = this.plazas[plaza];
		if(p.plazaLlena())
			throw new IllegalStateException("La plaza está llena.");
		
		
		//Contenedor FlatRack
		if(contenedor instanceof FlatRack) {
			this.colocarContenedorFlatRack(contenedor, plaza);
		}
		
		//Contenedor Refrigerado
		else if (contenedor instanceof Refrigerado) {
			//Comprobamos si la plaza esta vacia
			if (!this.plazas[plaza].plazaVacia())
				throw new IllegalStateException("La plaza no esta vacía. No podemos colocar el contenedor Refrigerado en el suelo");
			else {
				this.plazas[plaza].colocarContenedor(contenedor);
			}
		}
		
		//Contenedor Estandar
		else {
			this.plazas[plaza].colocarContenedor(contenedor);
		}
		
	}
	
	/**
	 * Metodo para colocar un contenedor de tipo FlatRack
	 * @param contenedor Contenedor a colocar
	 * @param plaza numero de plaza a colocar
	 * @throws IllegalStateException Si el numero de plazas totales de este muelle es demasiado pequeño
	 * @throws IllegalStateException Si la plaza no esta vacia, no podemos colocar el FlatRack.
	 * @throws IllegalStateException Si las plazas de la izquierda y derecha estan ocupadas
	 */
	private void colocarContenedorFlatRack(Contenedor contenedor, int plaza) {
		if(this.plazas.length < 2)
			throw new IllegalStateException("El numero de plazas totales de este muelle es demasiado pequeño");
		
		if (!this.plazas[plaza].plazaVacia())
			throw new IllegalStateException("La plaza no esta vacia, no podemos colocar el FlatRack");
		boolean sePuedeColocar = false;
		//Comprobamos la plaza de la izquierda
		if ((plaza-1) >= 0) {
			//Si la plaza de la izquierda esta vacia, se coloca el contenedor
			if (this.plazas[plaza-1].plazaVacia()) {
				sePuedeColocar = true;
				this.plazas[plaza].colocarContenedor(contenedor);
				this.plazas[plaza-1].colocarContenedor(contenedor);
			}
		}
		
		//La plaza de la derecha
		if((plaza+1) <= this.plazas.length && !sePuedeColocar) {
			if (this.plazas[plaza+1].plazaVacia()) {
				sePuedeColocar = true;
				this.plazas[plaza].colocarContenedor(contenedor);
				this.plazas[plaza+1].colocarContenedor(contenedor);
			}
		}
		
		//Si no se ha colocado (izquierda y derecha ocupadas)
		if(!sePuedeColocar) {
			throw new IllegalStateException("Las plazas de la izquierda y derecha estan ocupadas");
		}	
	
	}

	/**
	 * 1.1.12 Metodo que sirve para sacar un contenedor de una Plaza y desapilarlo si es posible
	 * @param contenedor Contenedor a desapilar de la plaza
	 * @throws IllegalArgumentException si el contenedor es nulo
	 * @throws IllegalStateException si el contenedor se encuentra en transito
	 * @throws IllegalStateException si el contenedor no se encuentra apilado en el muelle
	 * @throws IllegalStateException si el contenedor no se encuentra en la ultima plaza
	 */
	public void sacarContenedorDePlaza(Contenedor contenedor) {
		//Comprobamos que el contenedor no sea nulo o que esté en transito
		if (contenedor == null) 
			throw new IllegalArgumentException("El Contenedor no puede ser nulo");
		if (contenedor.contenedorEnTransito()) 
			throw new IllegalStateException("El Contenedor se encuentra en transito");
		
		//Comprobar si se encuentra en funcionamiento
		if(!this.estaOperativo())
			throw new IllegalStateException("El muelle se encuentra fuera de servicio");
		
		String c = contenedor.getCodigo();
		//Buscamos que el contenedor este en el muelle
	    if (plazaActual(c) == -1)
	    	throw new IllegalStateException("El contenedor no se encuentra apilado en este muelle");
	    
		//Buscamos el contenedor y desapilamos el contenedor si se encuentra en el ultimo nivel
		for (int i = 0; i<this.plazas.length; i++) {
			if (this.plazas[i].contieneContenedor(c)) {
		        if (this.plazas[i].estaEnUltimoNivel(c))
		        	//Desapilamos el contenedor
		        	this.plazas[i].desapilarContenedor();
		        
		        	//Comparamos si es un FlatRack (Desapilarlo por completo)
		        	if (contenedor instanceof FlatRack) {
		        		boolean desapilado = false;
		        		if (i-1>=0) {
		        			if(this.plazas[i].ultimoContenedor().getCodigo().equals(this.plazas[i-1].ultimoContenedor().getCodigo())) {
		        				this.plazas[i-1].desapilarContenedor();
		        				desapilado = true;
		        			}	
		        		}
		        		if (!desapilado) {
		        			this.plazas[i+1].desapilarContenedor();
		        		}
		        		
		        	}
		        else
		        	throw new IllegalStateException("El Contenedor no esta en el ultimo nivel, no se puede desapilar");
			}
	    }
	}
	
	/**
	 * Metodo para alternar si esta operativo o no (Test)
	 */
	public void alternarOperativo() {
		this.estadoActual = (this.estadoActual == estado.OPERATIVO) ? estado.FUERA_DE_SERVICIO : estado.OPERATIVO;
    }	
	
	/**
	 * Metodo que sirve para si un codigo de contenedor es correcto
	 * @param codigo Codigo del contenedor que se comprueba
	 * @return true si el codigo es correcto o false si no lo es
	 */
	private boolean comprobarCodigoContenedor(String codigo) {
		return (this.CODIGO_CONTENEDOR.matcher(codigo).matches()) ? true : false;	
	}
	
	/**
	 * Metodo que devuelve el identificador de un muelle. Este identificador sera de 2 digitos.
	 * @return identificador del muelle
	 */
	public String getIdentificador() {
		return this.identificador;
	}
	
	/**
	 * Metodo que indica si el Muelle esta operativo o no
	 * @return Si esta operativo (true) o no (false)
	 */
	public boolean estaOperativo() {
		return this.estadoActual == estado.OPERATIVO;
	}
	
	/**
	 * Metodo que devuelve el punto de coordenadas del muelle
	 * Este punto pertenece a la clase GPSCoordinate
	 * @return coordinates del muelle;
	 */
	public GPSCoordinate getGPSCoordinate() {
		return this.cord;
	}
	
	/**
	 * Metodo que devuelve la lista de las plazas 
	 * @return Lista de las plazas
	 */
	public Plaza[] plazas() {
		return this.plazas.clone();
	}
	
	/**
	 * Metodo que devuelve si el muelle puede usar el tren
	 * @return booleano con la utilidad del tren
	 */
	public boolean utilidadTren() {
		return this.utilidadTren;
	}
	
	/**
	 * Metodo que indica si el muelle esta alejado de un barco.
	 * @return booleano que indica si el muelle esta alejado o no de un barco
	 */
	public boolean alejadoBarco() {
		return this.alejadoBarco;
	}
}