/**
 * Copyright UVa 2024
 */
package es.markse;

import java.util.ArrayList;

/**
 * Implementacion de la clase PackCamionBarco, que representa un trayecto combinado de camión y barco.
 * @author javcalv
 * @author victorm
 */
public class PackCamionBarco extends Combinado {
	
	//Descuentos que se aplican
    private final double DESCUENTO = 0.85;

    /**
     * Constructor de la clase PackCamionBarco
     * @param muelleOrigen Muelle desde el que empieza el Trayecto
     * @param puertoOrigen Puerto desde el que empieza el Trayecto
     * @param fechaInicioTrayecto Fecha en la que comienza el Trayecto
     * @param muelleDestino Muelle en el que finaliza un Trayecto
     * @param puertoDestino Puerto en el que finaliza un Trayecto
     * @param fechaFinTrayecto Fecha en la que finaliza un Trayecto
     * @param trayectosCamion arrayList con los trayectos 
     * @param trayectosBarco
     */
    public PackCamionBarco(Muelle muelleOrigen, Puerto puertoOrigen, Fecha fechaInicioTrayecto, Muelle muelleDestino, 
    		Puerto puertoDestino, Fecha fechaFinTrayecto, ArrayList<Simple> trayectosSimples) {
    	super(muelleOrigen, puertoOrigen, fechaInicioTrayecto, muelleDestino, puertoDestino, fechaFinTrayecto);
    	
    	//Comprobamos si los trayectos no estan nulos (Debe de tener uno al menos)
    	if(estanVacios(trayectosSimples))
    		throw new IllegalArgumentException("Los trayectos en camion estan vacios");
    	
    	//Comprobamos si las clases son solo de tipo Camion o Tren
        if (!trayectosCorrectos(trayectosSimples))
        	throw new IllegalArgumentException("Las clases agregadas al ArrayList son invalidas");
        
        //Comprobar si hay un trayecto de barco y de camion al menos (Obligatorio por nuestra parte)
        if (!this.comprobarSiHayTrayectosCamion(trayectosSimples))
        	throw new IllegalArgumentException("En los trayectos no hay trayectos en camion (Es obligatorio)");
        if (!this.comprobarSiHayTrayectosBarco(trayectosSimples))
        	throw new IllegalArgumentException("En los trayectos no hay trayectos en tren (Es obligatorio)");
        
        //Comprobamos que el orden de los trayectos sea correcto (Destino de uno es el origen del siguiente)
        if (!this.ordenTrayectosCorrecto(trayectosSimples))
        	throw new IllegalArgumentException("El orden de los trayectos no es correcto. "
        			+ "Comprueba si el destino es el mismo que el origen del siguiente trayecto");
        
        this.trayectosSimples = trayectosSimples;
        compararTrayectosDeLaLista(trayectosSimples);
    	
    }

    /**
     * Metodo para comparar si las instancias de los trayectos simples son correctas
     * @param ts Arraylist con los Trayectos a comparar
     * @return true si son correctos o false si no lo son
     */
    private boolean trayectosCorrectos(ArrayList<Simple> ts) {
        for (Simple t : ts) {
            if (!(t instanceof TCamion || t instanceof TBarco)) {
                return false;
            }
        }
        return true;
    }
    
    /**
     * Metodo que comprueba si hay trayectos de barco en el pack
     * @param trayectos ArrayList de los trayectos simples a comprobar
     * @return true si hay trayectos de tren o false si no los hay
     */
    private boolean comprobarSiHayTrayectosBarco(ArrayList<Simple> trayectos) {
    	int trayectosBarco = 0;
    	for (Simple t : trayectos) {
    		if (t instanceof TBarco) trayectosBarco++;	
    	}
    	return (trayectosBarco == 0)? false : true;
    }
    
    /************************
     * METODOS DEL TRAYECTO *
     ************************/
    
	/**
     * Método que calcula el coste del trayecto combinado de camión y barco
     * @return Coste del trayecto en euros
     */
    @Override
    protected double costeTrayecto() {
    	double coste = 0;
    	for (Simple ts: this.trayectosSimples) {
    		if (ts instanceof TBarco) {
    			TBarco tb = (TBarco) ts;
    			coste += tb.costeTrayecto() * this.DESCUENTO;
    		}
    		else {
    			TCamion tc = (TCamion) ts;
    			coste += tc.costeTrayecto();
    		}
    	}
    	return coste;
    }
}