/**
 * Copyright UVa 2024
 */
package es.markse;
import java.util.ArrayList;

/**
 * Implementacion de la clase PackCamionTren, que representa un trayecto combinado de camión y tren.
 * @author javcalv
 * @author victorm
 */
public class PackCamionTren extends Combinado {
	
	//Reduccion de los precios
    private final int RED_COSTE_FIJO = 0;
    private final int RED_COSTE_POR_KILOMETRO = 10;

    /**
     * Constructor de la clase PackCamionTren
     * @param muelleOrigen Muelle desde el que empieza el Trayecto
     * @param puertoOrigen Puerto desde el que empieza el Trayecto
     * @param fechaInicioTrayecto Fecha en la que comienza el Trayecto
     * @param muelleDestino Muelle en el que finaliza un Trayecto
     * @param puertoDestino Puerto en el que finaliza un Trayecto
     * @param fechaFinTrayecto Fecha en la que finaliza un Trayecto
     * @param trayectosSimples ArrayList con los trayectos simples
     * @throws IllegalArgumentException Si la lista esta vacia
     * @throws IllegalArgumentException Si las clases agregadas en el ArrayList son invalidas
     * @throws IllegalArgumentException Si no se encuentra un solo trayecto de camion en la lista
     * @throws IllegalArgumentException Si no se encuentra un solo trayecto de tren en la lista
     * @throws IllegalArgumentException Si El orden de los trayectos no es correcto
     */
    public PackCamionTren(Muelle muelleOrigen, Puerto puertoOrigen, Fecha fechaInicioTrayecto, Muelle muelleDestino, 
    		Puerto puertoDestino, Fecha fechaFinTrayecto, ArrayList<Simple> trayectosSimples){
        super(muelleOrigen, puertoOrigen, fechaInicioTrayecto, muelleDestino, puertoDestino, fechaFinTrayecto);
        
        //Comprobamos que los trayectos no sean nulos/vacios
        if (estanVacios(trayectosSimples))
        	throw new IllegalArgumentException("Los trayectos no pueden ser nulos/vacios");
        
        //Comprobamos si las clases son solo de tipo Camion o Tren
        if (!trayectosCorrectos(trayectosSimples))
        	throw new IllegalArgumentException("Las clases agregadas al ArrayList son invalidas");
        	
        //Comprobar si hay un trayecto de barco y de camion al menos (Obligatorio por nuestra parte)
        if (!this.comprobarSiHayTrayectosCamion(trayectosSimples))
        	throw new IllegalArgumentException("En los trayectos no hay trayectos en camion (Es obligatorio)");
        if (!this.comprobarSiHayTrayectosTren(trayectosSimples))
        	throw new IllegalArgumentException("En los trayectos no hay trayectos en tren (Es obligatorio)");
        
        //Comprobamos que el orden de los trayectos sea correcto (Destino de uno es el origen del siguiente)
        if (!this.ordenTrayectosCorrecto(trayectosSimples))
        	throw new IllegalArgumentException("El orden de los trayectos no es correcto. "
        			+ "Comprueba si el destino es el mismo que el origen del siguiente trayecto");
        
        this.trayectosSimples = trayectosSimples;
        compararTrayectosDeLaLista(trayectosSimples);
        
    }
    
    /**
     * Metodo que comprueba si hay trayectos de tren en el pack
     * @param trayectos ArrayList de los trayectos simples a comprobar
     * @return true si hay trayectos de tren o false si no los hay
     */
    private boolean comprobarSiHayTrayectosTren(ArrayList<Simple> trayectos) {
    	int trayectosTren = 0;
    	for (Simple t : trayectos) {
    		if (t instanceof TTren) trayectosTren++;	
    	}
    	return (trayectosTren == 0)? false : true;
    }
    
    /**
     * Metodo para comparar si las instancias de los trayectos simples son correctas
     * @param ts Arraylist con los Trayectos a comparar
     * @return true si son correctos o false si no lo son
     */
    private boolean trayectosCorrectos(ArrayList<Simple> ts) {
        for (Simple t : ts) {
            if (!(t instanceof TCamion || t instanceof TTren)) {
                return false;
            }
        }
        return true;
    }
    
    
    /************************
     * METODOS DEL TRAYECTO *
     ************************/
    
    /**
     * Método que calcula el coste del trayecto combinado de camiónes y trenes
     * @return Coste del trayecto en euros
     */
    @Override
    protected double costeTrayecto() {
    	double coste = 0;
    	for (Simple ts : this.trayectosSimples) {
    		TVehiculoTierra tv = (TVehiculoTierra) ts;
    		tv.COSTE_FIJO = this.RED_COSTE_FIJO;
    		tv.COSTE_POR_KILOMETRO = this.RED_COSTE_POR_KILOMETRO;
    		coste += tv.costeTrayecto();	
    	}
    	return coste;
    }
}