/**
 * Copyright UVa 2024/2025
 */

package es.markse;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

import es.uva.inf.poo.maps.GPSCoordinate;

/**
 * Implementacion de la clase Puerto.
 * @author javcalv
 * @author victorm
 */
public class Puerto {
	private final String pais;
	private final String ciudad;
	private final List<Muelle> muelles = new ArrayList<>();
	private final Pattern PAIS_REGUEX = Pattern.compile("[A-Za-z]{2}");
	private final Pattern CIUDAD_REGUEX = Pattern.compile("[A-Za-z]{3}");
	
	
	/**
	 * Constructor del objeto Puerto.
	 * @param pais Pais al que pertenece el puerto (2 letras)
	 * @param ciudad Ciudad al que pertenece el puerto (3 letras)
	 */
	public Puerto(String pais, String ciudad) {
		if(pais == null || ciudad == null) 
			throw new IllegalArgumentException("El pais o la ciudad no pueden ser nulos");
		if(!comprobarIdentifiadores(pais.toUpperCase(), ciudad.toUpperCase()))
			throw new IllegalArgumentException("El pais o la ciudad no tiene el formato incorrecto. 'XX' pais y 'XXX' ciudad.");
		this.pais = pais.toUpperCase();
		this.ciudad = ciudad.toUpperCase();
	}
	
	/**
	 * Metodo que comprueba si el pais o la ciudad tienen el formato correcto mediante expresiones regulares.
	 * @param pais Pais del puerto a comprobar
	 * @param ciudad Ciudad del puerto a comprobar
	 * @return true si es el formto correcto o false si no lo es
	 */
	private boolean comprobarIdentifiadores(String pais, String ciudad) {
		return (PAIS_REGUEX.matcher(pais).matches() && CIUDAD_REGUEX.matcher(ciudad).matches()) ? true: false;
	}
	
	/**
	 * 1.1.1. Metodo que agrega un nuevo muelle al puerto. Se comprueba que ese muelle no se encuentre ya agregado.
	 * Si no lo esta, lo agrega, si ya se encuentra agregado, no.
	 * @param m objeto tipo Muelle
	 * @return true si el muelle se ha agregado, o false si el muelle ya esta agregado
	 * @throws IllegalArgumentException si el muelle es nulo
	 */
	public boolean anyadirMuelle(Muelle m) {
		if (m == null) 
			throw new IllegalArgumentException("Muelle no puede ser nulo");
		if (!this.comprobarMuelleEnPuerto(m))
			return this.muelles.add(m);
		return false;
	}
	
	/**
	 * 1.1.2 Método que elimina un muelle del puerto. Si encuentra uno con el mismo ID, lo elimina
	 * @param id : Identificador del muelle que se elimina
	 * @return true si se ha eliminado el muelle y false si no se ha eliminado ninguno (n)
	 * @throws IllegalArgumentException si el id es nulo;
	 */
	public boolean eliminarMuelle(String id) {
		if (id == null || id == "") 
			throw new IllegalArgumentException("Identificador no puede ser nulo o vacio");
		return this.muelles.removeIf(muelle -> muelle.getIdentificador().equals(id));
	}
	
	/**
	 * 1.1.3 Metodo que comprueba si un puerto esta Lleno o no
	 * @return true (si esta lleno) o false (si no lo esta)
	 */
	public boolean comprobarSiEstaLleno() {
		for(Muelle muelle : this.muelles) {
			if(muelle.plazasVacias() != 0) {
				return false;
			}
		}
		return true;
	}
	
	/**
	 * 1.1.4 Metodo que devuelve una lista con todos los muelles operativos del puerto
	 * @return lista con los muelles operativo
	 */
	public Muelle[] muellesOperativos() {
		List<Muelle> operativos = new ArrayList<>();
	    for (Muelle muelle : this.muelles) {
	        if (muelle.estaOperativo()) {
	            operativos.add(muelle);
	        }
	    }
	    return operativos.toArray(new Muelle[0]);
	}
	
	/**
	 * 1.1.5 Metodo que devuelve una lista con los muelles con espacio
	 * @return Lista con muelles con espacio
	 */
	public Muelle[] muellesConEspacio() {
		List<Muelle> conEspacio = new ArrayList<>();
		for (Muelle muelle : this.muelles) {
			if(muelle.plazasVacias() != 0) {
				conEspacio.add(muelle);
			}
		}
		return conEspacio.toArray(new Muelle[0]);
	}
	
	/**
	 * 1.1.6. Metodo que devuelve que muelles estas mas cerca e un punto GPS dado. Para ello se calculara la distancia que hay entre
	 * cada muelle y el punto dado, y se devolvera una lista con los muelles mas cercanos dados.
	 * @param puntoDado: Punto GPS dado, que puede ser el de un barco.
	 * @return Lista con los muelles mas cercanos a ese punto gps dado
	 */
	public Muelle[] muellesCercanoAPunto(GPSCoordinate puntoDado) {
		if (puntoDado == null) throw new IllegalArgumentException("El Punto no puede ser nulo");
	    List<Muelle> cercanos = new ArrayList<>();
	    //Inicializamos la distanciaMinima con el valor mas alto posible (la primera distancia sera siempre mas corta)
	    double distanciaMinima = Double.MAX_VALUE;
	    
	    //Recorremos todos los muelles y agregamos los que sean menor
	    for (Muelle muelle : this.muelles) {
	        double distancia = muelle.getGPSCoordinate().getDistanceTo(puntoDado);
	        if (distancia < distanciaMinima) {
	        	//Si encontramos un muelle nuevo mas cerca, se limpian los que ya estaban
	            cercanos.clear();
	            cercanos.add(muelle);
	            distanciaMinima = distancia;
	        }
	        else if (distancia == distanciaMinima) {
	            cercanos.add(muelle);
	        }
	    }
	    return cercanos.toArray(new Muelle[0]);
	}
	
	/**
	 * Metodo para comprobar si un muelle esta en ese puerto o no, para agregarle, 
	 * ya que solo puede haber un muelle con el mismo identificador para cada puerto.
	 * @param m obtejo Muelle
	 * @return true si esta, y false si no se encuentra en el puerto
	 * @throws IllegalArgumentException si el muelle es nulo
	 */
	public boolean comprobarMuelleEnPuerto(Muelle m) {
		if (m == null)
			throw new IllegalArgumentException("El muelle no puede ser nulo");
	    String id = m.getIdentificador();
	    GPSCoordinate cord = m.getGPSCoordinate();
	    for (Muelle muelle : this.muelles) {
	        if (id.equals(muelle.getIdentificador()) || (cord.getDistanceTo(muelle.getGPSCoordinate())==0)) {
	            return true;
	        }
	    }
	    return false;
	}
	
	/**
	 * Metodo que devuelve el pais del puerto
	 * @return pais del perto
	 */
	public String paisDelPuerto() {
		return this.pais;
	}
	
	/**
	 * Metodo que devuelve la ciudad del puerto
	 * @return ciudad del puerto
	 */
	public String ciudadDelPuerto() {
		return this.ciudad;
	}
	
	/**
	 * metodo que devuelve el identificador completo del puerto
	 * @return identificador del puerto completo
	 */
	public String identificadorPuerto() {
		return this.paisDelPuerto()+"-"+this.ciudadDelPuerto();
	}
}