/**
 * Copyright UVa 2024/2025
 */
package es.markse;

/**
 * Implementacion de la clase Trayecto
 * @author vicmtorm
 * @author javcalv
 */
public abstract class Trayecto{
	
	protected final Muelle muelleOrigen;
	protected final Puerto puertoOrigen;
	protected final Fecha fechaInicioTrayecto;
	protected final Muelle muelleDestino;
	protected final Puerto puertoDestino;
	protected final Fecha fechaFinTrayecto;
	
	/**
	 * Constructor del objeto Trayecto.
	 * @param muelleOrigen Muelle desde el que se e mpieza el Trayecto
	 * @param puertoOrigen Puerto desde el que se empieza el Trayecto
	 * @param fechaInicioTrayecto Fecha en la que comienza el Trayecto
	 * @param muelleDestino Muelle en el que finaliza un Trayecto
	 * @param puertoDestino Puerto en el que finaliza un Trayecto
	 * @param fechaFinTrayecto Fecha en la que finaliza un Trayecto
	 */
	public Trayecto (Muelle muelleOrigen, Puerto puertoOrigen, Fecha fechaInicioTrayecto, Muelle muelleDestino, Puerto puertoDestino, Fecha fechaFinTrayecto) {
		//Comprobamos si los valores no son nulos
		if(muelleOrigen == null || muelleDestino == null)
			throw new IllegalArgumentException("Los muelles no pueden ser nulos");
		if(puertoOrigen == null || puertoDestino == null)
			throw new IllegalArgumentException("Los puertos no pueden ser nulos");
		if(fechaInicioTrayecto == null || fechaFinTrayecto == null)
			throw new IllegalArgumentException("Las fechas no pueden ser nulas");
			
		//Comprobamos si los muelles estan operativos
		if (!muelleOrigen.estaOperativo())
			throw new IllegalStateException("El muelle de origen esta fuera de servicio");
		if (!muelleDestino.estaOperativo())
			throw new IllegalStateException("El muelle de destino esta fuera de servicio");
		
		//Comprobamos si los muelles pertenecen al puerto
		if (!puertoOrigen.comprobarMuelleEnPuerto(muelleOrigen))
			throw new IllegalStateException("El muelle no pertenece al puerto");
		if (!puertoDestino.comprobarMuelleEnPuerto(muelleDestino))
			throw new IllegalStateException("El muelle no pertenece al puerto");
		
		if(fechaInicioTrayecto.getDiasDesdeEpoch()>fechaFinTrayecto.getDiasDesdeEpoch())
			throw new IllegalArgumentException ("La Fecha del Inicio del Trayecto no puede ser posterior a la fecha del Fin del Trayecto");
		
		this.muelleOrigen=muelleOrigen;
		this.puertoOrigen=puertoOrigen;
		this.fechaInicioTrayecto=fechaInicioTrayecto;
		this.muelleDestino = muelleDestino;
		this.puertoDestino = puertoDestino;
		this.fechaFinTrayecto = fechaFinTrayecto;	
	}
	
	/*********************************************
	 * IMPLEMENTACION DE FUNCIONALIDADES BASICAS *
	 *********************************************/
	
	/**
	 * 1.1.21 Método que indica si la Fecha de Fin de Trayecto es superior a una Fecha Dada
	 * @param fechaDada Fecha de Introducida por el usuario
	 * @return True si la Fecha de Fin de Trayecto es Superior, y False si la Fecha de Fin de Trayecto no es Superior
	 * @throws IllegalArgumentException si la fecha es nula
	 */
	public boolean FechaFinTrayectoSuperior(Fecha fechaDada) {
		if (fechaDada == null) 
			throw new IllegalArgumentException("Fecha no puede ser nula");
		return (fechaFinTrayecto.getDiasDesdeEpoch()>fechaDada.getDiasDesdeEpoch()) ? false : true;
	}
	
	/**
	 * 1.1.22. Metodo que, introduciendo el coste diario y el coste por milla, devuelve el precio calculado en Euros de un Trayecto
	 * @return Precio del Trayecto Calculado en Euros
	 */
	public double precioTrayectoEnEuros() {
		return this.costeTrayecto();
	}
	
	/**
	 * Metodo abstracto para calcular el nuevo coste de 
	 * @return coste del trayecto
	 */
	protected abstract double costeTrayecto();
	
	/**
	 * 1.1.23 Método que devuelve la Distancia calculada en Millas Marinas entre 2 Muelles Distintos
	 * @return Distancia entre 2 Muelles Distintos calculada en Millas Marinas
	 */
	public double distanciaMillasMarinas() {
		return muelleOrigen.getGPSCoordinate().getDistanceTo(muelleDestino.getGPSCoordinate());
	}
	
	/**
	 * 1.1.24 Método que Proporciona Información acerca del Trayecto: Localidad y País de los Puertos de Origen y Destino, y Fechas de Inicio y Fin del Trayecto
	 * @return Información Sobre el Origen (Ciudad, Pais, Fecha Inicio) y el Destino (Ciudad, Pais, Fecha Fin)
	 */
	public String inforTrayecto() {
		String ciudadOrig = this.puertoOrigen.ciudadDelPuerto();
		String paisOrig = this.puertoOrigen.paisDelPuerto();
		String FechaInicio = this.fechaInicioTrayecto.aCadena();
		String ciudadDest = this.puertoDestino.ciudadDelPuerto();
		String paisDest = this.puertoDestino.paisDelPuerto();
		String FechaFin = this.fechaFinTrayecto.aCadena();
		return"INFORMACION SOBRE EL ORIGEN: \nCiudad Origen: "+ciudadOrig+"\nPais Origen: "+paisOrig+"\nInicio del Trayecto: "+FechaInicio+
				"\n\nINFORMACION SOBRE EL DESTINO: \nCiudad Destino: "+ciudadDest+"\nPais Destino: "+paisDest+"\nFin del Trayecto: "+FechaFin+"\n";
	}	

	
	/*************************
	 * OTRAS FUNCIONALIDADES *
	 *************************/
	
	/**
	 * Metodo que devuelve el numero de dias de un trayecto
	 */
	public int numeroDeDiasDeTrayecto() {
        return (this.fechaFinTrayecto.getDiasDesdeEpoch() - this.fechaInicioTrayecto.getDiasDesdeEpoch()) + 1;
    }
	
	/**
	 * Método que Devuelve el Muelle de Origen
	 * @return muelleOrigen
	 */
    public Muelle getMuelleOrigen() {
    	return this.muelleOrigen;
    }
	
	/**
	 * Método que Devuelve el Muelle de Destino
	 * @return muelleDestino
	 */
	public Muelle getMuelleDestino() {
		return this.muelleDestino;
	}
	
	/**
	 * Método que Devuelve el Puerto de Origen
	 * @return puertoOrigen
	 */
	public Puerto getPuertoOrigen() {
		return this.puertoOrigen;
	}
	
	/**
	 * Método que Devuelve el Puerto de Destino
	 * @return puertoDestino
	 */
	public Puerto getPuertoDestino() {
		return this.puertoDestino;
	}
	
	/**
	 * Método que Devuelve la Fecha del Inicio del Trayecto
	 * @return fechaInicioTrayecto
	 */
	public Fecha getFechaInicioTrayecto() {
		return this.fechaInicioTrayecto;
	}
	
	/**
	 * Método que Devuelve la Fecha del Fin del Trayecto
	 * @return fechaFinTrayecto
	 */
	public Fecha getFechaFinTrayecto() {
		return this.fechaFinTrayecto;
	}
}